/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include "OnScreenKeypad.h"

OnScreenKeypad::OnScreenKeypad() {
	set_flags(Gtk::NO_WINDOW);
	set_redraw_on_allocate(false);
	Target=NULL;

	char ButtonText[2];
	ButtonText[1]=0;
	for (ButtonText[0]='A'; ButtonText[0]<='Z'; ButtonText[0]++) {
		Gtk::Button *NewButton=new Gtk::Button();
		NewButton->set_label(ButtonText);
		on_add(NewButton);
	}
}

OnScreenKeypad::~OnScreenKeypad() {
	/* No need to remove gracefully since we are toast - just free the memory */
	for (std::vector<Gtk::Button*>::iterator Keyi=KeyButtons.begin(); Keyi!=KeyButtons.end(); Keyi++) {
		delete (*Keyi);
	}
}

void OnScreenKeypad::on_size_request(Gtk::Requisition* requisition) {
	Gtk::Requisition KeyReq;
	*requisition = Gtk::Requisition();

	requisition->height=0;
	requisition->width=0;

	for (std::vector<Gtk::Button*>::iterator Keyi=KeyButtons.begin(); Keyi!=KeyButtons.end(); Keyi++) {
		if ((*Keyi)->is_visible()) {
			KeyReq=(*Keyi)->size_request();
			if (KeyReq.height>requisition->height)
				requisition->height=KeyReq.height;
			requisition->width+=KeyReq.width;	/* A little padding appears to be needed, hence the +2 */
		}
	}
}

void OnScreenKeypad::on_size_allocate(Gtk::Allocation& allocation) {
	unsigned int XOffset=0;
	Gtk::Requisition KeyReq;

	set_allocation(allocation);

	for (std::vector<Gtk::Button*>::iterator Keyi=KeyButtons.begin(); Keyi!=KeyButtons.end(); Keyi++) {
		if ((*Keyi)->is_visible()) {
			Gtk::Allocation KeyAlloc;
			KeyReq=(*Keyi)->size_request();

			KeyAlloc.set_width(KeyReq.width);
			KeyAlloc.set_height(KeyReq.height);
			KeyAlloc.set_x(XOffset);
			KeyAlloc.set_y(allocation.get_y());
			(*Keyi)->size_allocate(KeyAlloc);
			XOffset+=KeyReq.width;
		}
	}
}

void OnScreenKeypad::forall_vfunc(gboolean include_internals, GtkCallback callback, gpointer callback_data) {
	for (std::vector<Gtk::Button*>::iterator Keyi=KeyButtons.begin(); Keyi!=KeyButtons.end(); Keyi++) {
	    callback((GtkWidget*)(*Keyi)->gobj(), callback_data);
	}
}

void OnScreenKeypad::on_add(Gtk::Button* child) {
	child->set_parent(*this);
	child->signal_clicked().connect(sigc::bind<Glib::ustring>(sigc::mem_fun(this,&OnScreenKeypad::on_Key_clicked),child->get_label()));
	child->show();
	KeyButtons.push_back(child);
}

void OnScreenKeypad::on_remove(Gtk::Button* child) {
	if (!child)
		return;

	for (std::vector<Gtk::Button*>::iterator Keyi=KeyButtons.begin(); Keyi!=KeyButtons.end(); Keyi++) {
		if ((*Keyi)==child) {
			if (child->is_visible()) {
				queue_resize();
			}
			child->unparent();
			KeyButtons.erase(Keyi);
			return;
		}
	}
}

GtkType OnScreenKeypad::child_type_vfunc() const {
	return Gtk::Button::get_type();
}

void OnScreenKeypad::on_Key_clicked(Glib::ustring Text) {
	int OldPosition;
	if (!Target)
		return;

	Target->delete_selection();
	OldPosition=Target->get_position();
	Target->insert_text(Text,Text.size(),OldPosition);
	Target->grab_focus();
	Target->set_position(OldPosition);
}

void OnScreenKeypad::set_target(Gtk::Entry *target) {
	Target=target;
}
