/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef OVERWRITEFILEDIALOG_H_
#define OVERWRITEFILEDIALOG_H_

#include <string>
#include <gtkmm/dialog.h>
#include <gtkmm/label.h>
#include <gtkmm/button.h>
#include <libglademm.h>

/* This class is designed to be instantiated once per group of similar file
 * operations.  The Run() method should be executed repeatedly for each conflicting
 * file.  Delete when the group of file operations are done
 */

class OverwriteFileDialog
{
public:
	enum Do_enum {
		DO_ASK,
		DO_KEEPALL,
		DO_OVERWRITEALL
	};

	/* The response given as a result of Run() */
	enum Response_enum {
		RESPONSE_CANCEL,
		RESPONSE_KEEP,
		RESPONSE_OVERWRITE,
	};

	/* The internal responses to button clicks */
	enum Response_Internal_enum {
		RESPONSE_INTERNAL_CANCEL,
		RESPONSE_INTERNAL_KEEP,
		RESPONSE_INTERNAL_OVERWRITE,
		RESPONSE_INTERNAL_KEEPALL,
		RESPONSE_INTERNAL_OVERWRITEALL
	};

	OverwriteFileDialog(std::string *GladeFilename);
	virtual ~OverwriteFileDialog();

	enum Response_enum Run(std::string Path);
	void Reset();
	void SetPath(std::string Path);

private:
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Dialog *GtkDialog;
	Gtk::Label *PathLabel;

	enum Do_enum Do;
};

#endif /*OVERWRITEFILEDIALOG_H_*/
